@@ -354,7 +354,7 @@ def popen(cmd, mode='r', bufsize=None):
 def _norm_version(version,build=''):
 
     """ Normalize the version and build strings and return a single
-        vesion string using the format major.minor.build (or patchlevel).
+        version string using the format major.minor.build (or patchlevel).
     """
     l = string.split(version,'.')
     if build:
@@ -601,7 +601,8 @@ def mac_ver(release='',versioninfo=('','',''),machine=''):
         versioninfo = (version,stage,nonrel)
     if sysa:
         machine = {0x1: '68k',
-                   0x2: 'PowerPC'}.get(sysa,'')
+                   0x2: 'PowerPC',
+                   0xa: 'i386'}.get(sysa,'')
     return release,versioninfo,machine
 
 def _java_getprop(name,default):
