@@ -1,52 +1,66 @@
-"""RFC-822 message manipulation class.
+"""RFC 2822 message manipulation.
 
-XXX This is only a very rough sketch of a full RFC-822 parser;
-in particular the tokenizing of addresses does not adhere to all the
-quoting rules.
+Note: This is only a very rough sketch of a full RFC-822 parser; in particular
+the tokenizing of addresses does not adhere to all the quoting rules.
+
+Note: RFC 2822 is a long awaited update to RFC 822.  This module should
+conform to RFC 2822, and is thus mis-named (it's not worth renaming it).  Some
+effort at RFC 2822 updates have been made, but a thorough audit has not been
+performed.  Consider any RFC 2822 non-conformance to be a bug.
+
+    RFC 2822: http://www.faqs.org/rfcs/rfc2822.html
+    RFC 822 : http://www.faqs.org/rfcs/rfc822.html (obsolete)
 
 Directions for use:
 
 To create a Message object: first open a file, e.g.:
+
   fp = open(file, 'r')
+
 You can use any other legal way of getting an open file object, e.g. use
-sys.stdin or call os.popen().
-Then pass the open file object to the Message() constructor:
+sys.stdin or call os.popen().  Then pass the open file object to the Message()
+constructor:
+
   m = Message(fp)
 
-This class can work with any input object that supports a readline
-method.  If the input object has seek and tell capability, the
-rewindbody method will work; also illegal lines will be pushed back
-onto the input stream.  If the input object lacks seek but has an
-`unread' method that can push back a line of input, Message will use
-that to push back illegal lines.  Thus this class can be used to parse
-messages coming from a buffered stream.
+This class can work with any input object that supports a readline method.  If
+the input object has seek and tell capability, the rewindbody method will
+work; also illegal lines will be pushed back onto the input stream.  If the
+input object lacks seek but has an `unread' method that can push back a line
+of input, Message will use that to push back illegal lines.  Thus this class
+can be used to parse messages coming from a buffered stream.
 
-The optional `seekable' argument is provided as a workaround for
-certain stdio libraries in which tell() discards buffered data before
-discovering that the lseek() system call doesn't work.  For maximum
-portability, you should set the seekable argument to zero to prevent
-that initial \code{tell} when passing in an unseekable object such as
-a a file object created from a socket object.  If it is 1 on entry --
-which it is by default -- the tell() method of the open file object is
-called once; if this raises an exception, seekable is reset to 0.  For 
-other nonzero values of seekable, this test is not made.
+The optional `seekable' argument is provided as a workaround for certain stdio
+libraries in which tell() discards buffered data before discovering that the
+lseek() system call doesn't work.  For maximum portability, you should set the
+seekable argument to zero to prevent that initial \code{tell} when passing in
+an unseekable object such as a a file object created from a socket object.  If
+it is 1 on entry -- which it is by default -- the tell() method of the open
+file object is called once; if this raises an exception, seekable is reset to
+0.  For other nonzero values of seekable, this test is not made.
 
 To get the text of a particular header there are several methods:
+
   str = m.getheader(name)
   str = m.getrawheader(name)
-where name is the name of the header, e.g. 'Subject'.
-The difference is that getheader() strips the leading and trailing
-whitespace, while getrawheader() doesn't.  Both functions retain
-embedded whitespace (including newlines) exactly as they are
-specified in the header, and leave the case of the text unchanged.
+
+where name is the name of the header, e.g. 'Subject'.  The difference is that
+getheader() strips the leading and trailing whitespace, while getrawheader()
+doesn't.  Both functions retain embedded whitespace (including newlines)
+exactly as they are specified in the header, and leave the case of the text
+unchanged.
 
 For addresses and address lists there are functions
-  realname, mailaddress = m.getaddr(name) and
+
+  realname, mailaddress = m.getaddr(name)
   list = m.getaddrlist(name)
+
 where the latter returns a list of (realname, mailaddr) tuples.
 
 There is also a method
+
   time = m.getdate(name)
+
 which parses a Date-like field and returns a time-compatible tuple,
 i.e. a tuple such as returned by time.localtime() or accepted by
 time.mktime().
@@ -57,16 +71,16 @@ There are also some utility functions here.
 """
 # Cleanup and extensions by Eric S. Raymond <esr@thyrsus.com>
 
-import string
 import time
 
+__all__ = ["Message","AddressList","parsedate","parsedate_tz","mktime_tz"]
 
 _blanklines = ('\r\n', '\n')            # Optimization for islast()
 
 
 class Message:
-    """Represents a single RFC-822-compliant message."""
-    
+    """Represents a single RFC 2822-compliant message."""
+
     def __init__(self, fp, seekable = 1):
         """Initialize the class instance and read the headers."""
         if seekable == 1:
@@ -74,7 +88,7 @@ class Message:
             # (and then assume seek() works, too)
             try:
                 fp.tell()
-            except:
+            except (AttributeError, IOError):
                 seekable = 0
             else:
                 seekable = 1
@@ -96,28 +110,27 @@ class Message:
                 self.startofbody = self.fp.tell()
             except IOError:
                 self.seekable = 0
-    
+
     def rewindbody(self):
         """Rewind the file to the start of the body (if seekable)."""
         if not self.seekable:
             raise IOError, "unseekable file"
         self.fp.seek(self.startofbody)
-    
+
     def readheaders(self):
         """Read header lines.
-        
-        Read header lines up to the entirely blank line that
-        terminates them.  The (normally blank) line that ends the
-        headers is skipped, but not included in the returned list.
-        If a non-header line ends the headers, (which is an error),
-        an attempt is made to backspace over it; it is never
-        included in the returned list.
-        
-        The variable self.status is set to the empty string if all
-        went well, otherwise it is an error message.
-        The variable self.headers is a completely uninterpreted list
-        of lines contained in the header (so printing them will
-        reproduce the header exactly as it appears in the file).
+
+        Read header lines up to the entirely blank line that terminates them.
+        The (normally blank) line that ends the headers is skipped, but not
+        included in the returned list.  If a non-header line ends the headers,
+        (which is an error), an attempt is made to backspace over it; it is
+        never included in the returned list.
+
+        The variable self.status is set to the empty string if all went well,
+        otherwise it is an error message.  The variable self.headers is a
+        completely uninterpreted list of lines contained in the header (so
+        printing them will reproduce the header exactly as it appears in the
+        file).
         """
         self.dict = {}
         self.unixfrom = ''
@@ -132,21 +145,25 @@ class Message:
             tell = self.fp.tell
         while 1:
             if tell:
-                startofline = tell()
+                try:
+                    startofline = tell()
+                except IOError:
+                    startofline = tell = None
+                    self.seekable = 0
             line = self.fp.readline()
             if not line:
                 self.status = 'EOF in headers'
                 break
             # Skip unix From name time lines
-            if firstline and line[:5] == 'From ':
+            if firstline and line.startswith('From '):
                 self.unixfrom = self.unixfrom + line
                 continue
             firstline = 0
             if headerseen and line[0] in ' \t':
                 # It's a continuation line.
                 list.append(line)
-                x = (self.dict[headerseen] + "\n " + string.strip(line))
-                self.dict[headerseen] = string.strip(x)
+                x = (self.dict[headerseen] + "\n " + line.strip())
+                self.dict[headerseen] = x.strip()
                 continue
             elif self.iscomment(line):
                 # It's a comment.  Ignore it.
@@ -158,7 +175,7 @@ class Message:
             if headerseen:
                 # It's a legal header line, save it.
                 list.append(line)
-                self.dict[headerseen] = string.strip(line[len(headerseen)+1:])
+                self.dict[headerseen] = line[len(headerseen)+1:].strip()
                 continue
             else:
                 # It's not a header line; throw it back and stop here.
@@ -179,106 +196,100 @@ class Message:
         """Determine whether a given line is a legal header.
 
         This method should return the header name, suitably canonicalized.
-        You may override this method in order to use Message parsing
-        on tagged data in RFC822-like formats with special header formats.
+        You may override this method in order to use Message parsing on tagged
+        data in RFC 2822-like formats with special header formats.
         """
-        i = string.find(line, ':')
+        i = line.find(':')
         if i > 0:
-            return string.lower(line[:i])
+            return line[:i].lower()
         else:
             return None
-    
+
     def islast(self, line):
-        """Determine whether a line is a legal end of RFC-822 headers.
-        
-        You may override this method if your application wants
-        to bend the rules, e.g. to strip trailing whitespace,
-        or to recognize MH template separators ('--------').
-        For convenience (e.g. for code reading from sockets) a
-        line consisting of \r\n also matches.                
+        """Determine whether a line is a legal end of RFC 2822 headers.
+
+        You may override this method if your application wants to bend the
+        rules, e.g. to strip trailing whitespace, or to recognize MH template
+        separators ('--------').  For convenience (e.g. for code reading from
+        sockets) a line consisting of \r\n also matches.
         """
         return line in _blanklines
 
     def iscomment(self, line):
         """Determine whether a line should be skipped entirely.
 
-        You may override this method in order to use Message parsing
-        on tagged data in RFC822-like formats that support embedded
-        comments or free-text data.
+        You may override this method in order to use Message parsing on tagged
+        data in RFC 2822-like formats that support embedded comments or
+        free-text data.
         """
-        return None
-    
+        return False
+
     def getallmatchingheaders(self, name):
         """Find all header lines matching a given header name.
-        
-        Look through the list of headers and find all lines
-        matching a given header name (and their continuation
-        lines).  A list of the lines is returned, without
-        interpretation.  If the header does not occur, an
-        empty list is returned.  If the header occurs multiple
-        times, all occurrences are returned.  Case is not
-        important in the header name.
+
+        Look through the list of headers and find all lines matching a given
+        header name (and their continuation lines).  A list of the lines is
+        returned, without interpretation.  If the header does not occur, an
+        empty list is returned.  If the header occurs multiple times, all
+        occurrences are returned.  Case is not important in the header name.
         """
-        name = string.lower(name) + ':'
+        name = name.lower() + ':'
         n = len(name)
         list = []
         hit = 0
         for line in self.headers:
-            if string.lower(line[:n]) == name:
+            if line[:n].lower() == name:
                 hit = 1
-            elif line[:1] not in string.whitespace:
+            elif not line[:1].isspace():
                 hit = 0
             if hit:
                 list.append(line)
         return list
-    
+
     def getfirstmatchingheader(self, name):
         """Get the first header line matching name.
-        
-        This is similar to getallmatchingheaders, but it returns
-        only the first matching header (and its continuation
-        lines).
+
+        This is similar to getallmatchingheaders, but it returns only the
+        first matching header (and its continuation lines).
         """
-        name = string.lower(name) + ':'
+        name = name.lower() + ':'
         n = len(name)
         list = []
         hit = 0
         for line in self.headers:
             if hit:
-                if line[:1] not in string.whitespace:
+                if not line[:1].isspace():
                     break
-            elif string.lower(line[:n]) == name:
+            elif line[:n].lower() == name:
                 hit = 1
             if hit:
                 list.append(line)
         return list
-    
+
     def getrawheader(self, name):
         """A higher-level interface to getfirstmatchingheader().
-        
-        Return a string containing the literal text of the
-        header but with the keyword stripped.  All leading,
-        trailing and embedded whitespace is kept in the
-        string, however.
-        Return None if the header does not occur.
+
+        Return a string containing the literal text of the header but with the
+        keyword stripped.  All leading, trailing and embedded whitespace is
+        kept in the string, however.  Return None if the header does not
+        occur.
         """
-        
+
         list = self.getfirstmatchingheader(name)
         if not list:
             return None
         list[0] = list[0][len(name) + 1:]
-        return string.joinfields(list, '')
-    
+        return ''.join(list)
+
     def getheader(self, name, default=None):
         """Get the header value for a name.
-        
-        This is the normal interface: it returns a stripped
-        version of the header value for a given header name,
-        or None if it doesn't exist.  This uses the dictionary
-        version which finds the *last* such header.
+
+        This is the normal interface: it returns a stripped version of the
+        header value for a given header name, or None if it doesn't exist.
+        This uses the dictionary version which finds the *last* such header.
         """
         try:
-            return self.dict[string.lower(name)]
+            return self.dict[name.lower()]
         except KeyError:
             return default
     get = getheader
@@ -286,32 +297,31 @@ class Message:
     def getheaders(self, name):
         """Get all values for a header.
 
-        This returns a list of values for headers given more than once;
-        each value in the result list is stripped in the same way as the
-        result of getheader().  If the header is not given, return an
-        empty list.
+        This returns a list of values for headers given more than once; each
+        value in the result list is stripped in the same way as the result of
+        getheader().  If the header is not given, return an empty list.
         """
         result = []
         current = ''
         have_header = 0
         for s in self.getallmatchingheaders(name):
-            if s[0] in string.whitespace:
+            if s[0].isspace():
                 if current:
-                    current = "%s\n %s" % (current, string.strip(s))
+                    current = "%s\n %s" % (current, s.strip())
                 else:
-                    current = string.strip(s)
+                    current = s.strip()
             else:
                 if have_header:
                     result.append(current)
-                current = string.strip(s[string.find(s, ":") + 1:])
+                current = s[s.find(":") + 1:].strip()
                 have_header = 1
         if have_header:
             result.append(current)
         return result
-    
+
     def getaddr(self, name):
         """Get a single address from a header, as a tuple.
-        
+
         An example return value:
         ('Guido van Rossum', 'guido@cwi.nl')
         """
@@ -321,14 +331,13 @@ class Message:
             return alist[0]
         else:
             return (None, None)
-    
+
     def getaddrlist(self, name):
         """Get a list of addresses from a header.
 
         Retrieves a list of addresses from a header, where each address is a
         tuple as returned by getaddr().  Scans all named headers, so it works
         properly with multiple To: or Cc: headers for example.
-
         """
         raw = []
         for h in self.getallmatchingheaders(name):
@@ -337,68 +346,67 @@ class Message:
             else:
                 if raw:
                     raw.append(', ')
-                i = string.find(h, ':')
+                i = h.find(':')
                 if i > 0:
                     addr = h[i+1:]
                 raw.append(addr)
-        alladdrs = string.join(raw, '')
-        a = AddrlistClass(alladdrs)
-        return a.getaddrlist()
-    
+        alladdrs = ''.join(raw)
+        a = AddressList(alladdrs)
+        return a.addresslist
+
     def getdate(self, name):
         """Retrieve a date field from a header.
-        
-        Retrieves a date field from the named header, returning
-        a tuple compatible with time.mktime().
+
+        Retrieves a date field from the named header, returning a tuple
+        compatible with time.mktime().
         """
         try:
             data = self[name]
         except KeyError:
             return None
         return parsedate(data)
-    
+
     def getdate_tz(self, name):
         """Retrieve a date field from a header as a 10-tuple.
-        
-        The first 9 elements make up a tuple compatible with
-        time.mktime(), and the 10th is the offset of the poster's
-        time zone from GMT/UTC.
+
+        The first 9 elements make up a tuple compatible with time.mktime(),
+        and the 10th is the offset of the poster's time zone from GMT/UTC.
         """
         try:
             data = self[name]
         except KeyError:
             return None
         return parsedate_tz(data)
-    
-    
+
+
     # Access as a dictionary (only finds *last* header of each type):
-    
+
     def __len__(self):
         """Get the number of headers in a message."""
         return len(self.dict)
-    
+
     def __getitem__(self, name):
         """Get a specific header, as from a dictionary."""
-        return self.dict[string.lower(name)]
+        return self.dict[name.lower()]
 
     def __setitem__(self, name, value):
         """Set the value of a header.
 
-        Note: This is not a perfect inversion of __getitem__, because 
-        any changed headers get stuck at the end of the raw-headers list
-        rather than where the altered header was.
+        Note: This is not a perfect inversion of __getitem__, because any
+        changed headers get stuck at the end of the raw-headers list rather
+        than where the altered header was.
         """
         del self[name] # Won't fail if it doesn't exist
-        self.dict[string.lower(name)] = value
+        self.dict[name.lower()] = value
         text = name + ": " + value
-        lines = string.split(text, "\n")
+        lines = text.split("\n")
         for line in lines:
             self.headers.append(line + "\n")
-    
+
     def __delitem__(self, name):
         """Delete all occurrences of a specific header, if it is present."""
-        name = string.lower(name)
-        if not self.dict.has_key(name):
+        name = name.lower()
+        if not name in self.dict:
             return
         del self.dict[name]
         name = name + ':'
@@ -407,40 +415,55 @@ class Message:
         hit = 0
         for i in range(len(self.headers)):
             line = self.headers[i]
-            if string.lower(line[:n]) == name:
+            if line[:n].lower() == name:
                 hit = 1
-            elif line[:1] not in string.whitespace:
+            elif not line[:1].isspace():
                 hit = 0
             if hit:
                 list.append(i)
-        list.reverse()
-        for i in list:
+        for i in reversed(list):
             del self.headers[i]
 
+    def setdefault(self, name, default=""):
+        lowername = name.lower()
+        if lowername in self.dict:
+            return self.dict[lowername]
+        else:
+            text = name + ": " + default
+            lines = text.split("\n")
+            for line in lines:
+                self.headers.append(line + "\n")
+            self.dict[lowername] = default
+            return default
+
     def has_key(self, name):
         """Determine whether a message contains the named header."""
-        return self.dict.has_key(string.lower(name))
-    
+        return name.lower() in self.dict
+
+    def __contains__(self, name):
+        """Determine whether a message contains the named header."""
+        return name.lower() in self.dict
+
+    def __iter__(self):
+        return iter(self.dict)
+
     def keys(self):
         """Get all of a message's header field names."""
         return self.dict.keys()
-    
+
     def values(self):
         """Get all of a message's header field values."""
         return self.dict.values()
-    
+
     def items(self):
         """Get all of a message's headers.
-        
+
         Returns a list of name, value tuples.
         """
         return self.dict.items()
 
     def __str__(self):
-        str = ''
-        for hdr in self.headers:
-            str = str + hdr
-        return str
+        return ''.join(self.headers)
 
 
 # Utility functions
@@ -453,28 +476,22 @@ class Message:
 def unquote(str):
     """Remove quotes from a string."""
     if len(str) > 1:
-        if str[0] == '"' and str[-1:] == '"':
-            return str[1:-1]
-        if str[0] == '<' and str[-1:] == '>':
+        if str.startswith('"') and str.endswith('"'):
+            return str[1:-1].replace('\\\\', '\\').replace('\\"', '"')
+        if str.startswith('<') and str.endswith('>'):
             return str[1:-1]
     return str
 
 
 def quote(str):
     """Add quotes around a string."""
-    return '"%s"' % string.join(
-    string.split(
-    string.join(
-    string.split(str, '\\'),
-    '\\\\'),
-    '"'),
-    '\\"')
+    return str.replace('\\', '\\\\').replace('"', '\\"')
 
 
 def parseaddr(address):
     """Parse an address into a (realname, mailaddr) tuple."""
-    a = AddrlistClass(address)
-    list = a.getaddrlist()
+    a = AddressList(address)
+    list = a.addresslist
     if not list:
         return (None, None)
     else:
@@ -483,28 +500,34 @@ def parseaddr(address):
 
 class AddrlistClass:
     """Address parser class by Ben Escoto.
-    
+
     To understand what this class does, it helps to have a copy of
-    RFC-822 in front of you.
+    RFC 2822 in front of you.
+
+    http://www.faqs.org/rfcs/rfc2822.html
 
     Note: this class interface is deprecated and may be removed in the future.
     Use rfc822.AddressList instead.
     """
-    
+
     def __init__(self, field):
         """Initialize a new instance.
-        
-        `field' is an unparsed address header field, containing
-        one or more addresses.
+
+        `field' is an unparsed address header field, containing one or more
+        addresses.
         """
         self.specials = '()<>@,:;.\"[]'
         self.pos = 0
         self.LWS = ' \t'
         self.CR = '\r\n'
         self.atomends = self.specials + self.LWS + self.CR
+        # Note that RFC 2822 now specifies `.' as obs-phrase, meaning that it
+        # is obsolete syntax.  RFC 2822 requires that we recognize obsolete
+        # syntax, so allow dots in phrases.
+        self.phraseends = self.atomends.replace('.', '')
         self.field = field
         self.commentlist = []
-    
+
     def gotonext(self):
         """Parse up to the start of the next address."""
         while self.pos < len(self.field):
@@ -513,46 +536,50 @@ class AddrlistClass:
             elif self.field[self.pos] == '(':
                 self.commentlist.append(self.getcomment())
             else: break
-    
+
     def getaddrlist(self):
         """Parse all addresses.
-        
+
         Returns a list containing all of the addresses.
         """
-        ad = self.getaddress()
-        if ad:
-            return ad + self.getaddrlist()
-        else: return []
-    
+        result = []
+        while 1:
+            ad = self.getaddress()
+            if ad:
+                result += ad
+            else:
+                break
+        return result
+
     def getaddress(self):
         """Parse the next address."""
         self.commentlist = []
         self.gotonext()
-        
+
         oldpos = self.pos
         oldcl = self.commentlist
         plist = self.getphraselist()
-        
+
         self.gotonext()
         returnlist = []
-        
+
         if self.pos >= len(self.field):
             # Bad email address technically, no domain.
             if plist:
-                returnlist = [(string.join(self.commentlist), plist[0])]
-            
+                returnlist = [(' '.join(self.commentlist), plist[0])]
+
         elif self.field[self.pos] in '.@':
             # email address is just an addrspec
             # this isn't very efficient since we start over
             self.pos = oldpos
             self.commentlist = oldcl
             addrspec = self.getaddrspec()
-            returnlist = [(string.join(self.commentlist), addrspec)]
-            
+            returnlist = [(' '.join(self.commentlist), addrspec)]
+
         elif self.field[self.pos] == ':':
             # address is a group
             returnlist = []
-            
+
             fieldlen = len(self.field)
             self.pos = self.pos + 1
             while self.pos < len(self.field):
@@ -561,39 +588,39 @@ class AddrlistClass:
                     self.pos = self.pos + 1
                     break
                 returnlist = returnlist + self.getaddress()
-            
+
         elif self.field[self.pos] == '<':
             # Address is a phrase then a route addr
             routeaddr = self.getrouteaddr()
-            
+
             if self.commentlist:
-                returnlist = [(string.join(plist) + ' (' + \
-                         string.join(self.commentlist) + ')', routeaddr)]
-            else: returnlist = [(string.join(plist), routeaddr)]
-            
+                returnlist = [(' '.join(plist) + ' (' + \
+                         ' '.join(self.commentlist) + ')', routeaddr)]
+            else: returnlist = [(' '.join(plist), routeaddr)]
+
         else:
             if plist:
-                returnlist = [(string.join(self.commentlist), plist[0])]
+                returnlist = [(' '.join(self.commentlist), plist[0])]
             elif self.field[self.pos] in self.specials:
                 self.pos = self.pos + 1
-        
+
         self.gotonext()
         if self.pos < len(self.field) and self.field[self.pos] == ',':
             self.pos = self.pos + 1
         return returnlist
-    
+
     def getrouteaddr(self):
         """Parse a route address (Return-path value).
-        
+
         This method just skips all the route stuff and returns the addrspec.
         """
         if self.field[self.pos] != '<':
             return
-        
+
         expectroute = 0
         self.pos = self.pos + 1
         self.gotonext()
-        adlist = None
+        adlist = ""
         while self.pos < len(self.field):
             if expectroute:
                 self.getdomain()
@@ -606,19 +633,18 @@ class AddrlistClass:
                 expectroute = 1
             elif self.field[self.pos] == ':':
                 self.pos = self.pos + 1
-                expectaddrspec = 1
             else:
                 adlist = self.getaddrspec()
                 self.pos = self.pos + 1
                 break
             self.gotonext()
-        
+
         return adlist
-    
+
     def getaddrspec(self):
-        """Parse an RFC-822 addr-spec."""
+        """Parse an RFC 2822 addr-spec."""
         aslist = []
-        
+
         self.gotonext()
         while self.pos < len(self.field):
             if self.field[self.pos] == '.':
@@ -630,15 +656,15 @@ class AddrlistClass:
                 break
             else: aslist.append(self.getatom())
             self.gotonext()
-        
+
         if self.pos >= len(self.field) or self.field[self.pos] != '@':
-            return string.join(aslist, '')
-        
+            return ''.join(aslist)
+
         aslist.append('@')
         self.pos = self.pos + 1
         self.gotonext()
-        return string.join(aslist, '') + self.getdomain()
-    
+        return ''.join(aslist) + self.getdomain()
+
     def getdomain(self):
         """Get the complete domain name from an address."""
         sdlist = []
@@ -655,24 +681,24 @@ class AddrlistClass:
             elif self.field[self.pos] in self.atomends:
                 break
             else: sdlist.append(self.getatom())
-        return string.join(sdlist, '')
-    
+        return ''.join(sdlist)
+
     def getdelimited(self, beginchar, endchars, allowcomments = 1):
         """Parse a header fragment delimited by special characters.
-        
-        `beginchar' is the start character for the fragment.
-        If self is not looking at an instance of `beginchar' then
-        getdelimited returns the empty string.
-        
+
+        `beginchar' is the start character for the fragment.  If self is not
+        looking at an instance of `beginchar' then getdelimited returns the
+        empty string.
+
         `endchars' is a sequence of allowable end-delimiting characters.
         Parsing stops when one of these is encountered.
-        
-        If `allowcomments' is non-zero, embedded RFC-822 comments
-        are allowed within the parsed fragment.
+
+        If `allowcomments' is non-zero, embedded RFC 2822 comments are allowed
+        within the parsed fragment.
         """
         if self.field[self.pos] != beginchar:
             return ''
-        
+
         slist = ['']
         quote = 0
         self.pos = self.pos + 1
@@ -685,47 +711,55 @@ class AddrlistClass:
                 break
             elif allowcomments and self.field[self.pos] == '(':
                 slist.append(self.getcomment())
+                continue        # have already advanced pos from getcomment
             elif self.field[self.pos] == '\\':
                 quote = 1
             else:
                 slist.append(self.field[self.pos])
             self.pos = self.pos + 1
-        
-        return string.join(slist, '')
-    
+
+        return ''.join(slist)
+
     def getquote(self):
         """Get a quote-delimited fragment from self's field."""
         return self.getdelimited('"', '"\r', 0)
-    
+
     def getcomment(self):
         """Get a parenthesis-delimited fragment from self's field."""
         return self.getdelimited('(', ')\r', 1)
-    
+
     def getdomainliteral(self):
-        """Parse an RFC-822 domain-literal."""
+        """Parse an RFC 2822 domain-literal."""
         return '[%s]' % self.getdelimited('[', ']\r', 0)
-    
-    def getatom(self):
-        """Parse an RFC-822 atom."""
+
+    def getatom(self, atomends=None):
+        """Parse an RFC 2822 atom.
+
+        Optional atomends specifies a different set of end token delimiters
+        (the default is to use self.atomends).  This is used e.g. in
+        getphraselist() since phrase endings must not include the `.' (which
+        is legal in phrases)."""
         atomlist = ['']
-        
+        if atomends is None:
+            atomends = self.atomends
+
         while self.pos < len(self.field):
-            if self.field[self.pos] in self.atomends:
+            if self.field[self.pos] in atomends:
                 break
             else: atomlist.append(self.field[self.pos])
             self.pos = self.pos + 1
-        
-        return string.join(atomlist, '')
-    
+
+        return ''.join(atomlist)
+
     def getphraselist(self):
-        """Parse a sequence of RFC-822 phrases.
-        
-        A phrase is a sequence of words, which are in turn either
-        RFC-822 atoms or quoted-strings.  Phrases are canonicalized
-        by squeezing all runs of continuous whitespace into one space.
+        """Parse a sequence of RFC 2822 phrases.
+
+        A phrase is a sequence of words, which are in turn either RFC 2822
+        atoms or quoted-strings.  Phrases are canonicalized by squeezing all
+        runs of continuous whitespace into one space.
         """
         plist = []
-        
+
         while self.pos < len(self.field):
             if self.field[self.pos] in self.LWS:
                 self.pos = self.pos + 1
@@ -733,14 +767,15 @@ class AddrlistClass:
                 plist.append(self.getquote())
             elif self.field[self.pos] == '(':
                 self.commentlist.append(self.getcomment())
-            elif self.field[self.pos] in self.atomends:
+            elif self.field[self.pos] in self.phraseends:
                 break
-            else: plist.append(self.getatom())
-        
+            else:
+                plist.append(self.getatom(self.phraseends))
+
         return plist
 
 class AddressList(AddrlistClass):
-    """An AddressList encapsulates a list of parsed RFC822 addresses."""
+    """An AddressList encapsulates a list of parsed RFC 2822 addresses."""
     def __init__(self, field):
         AddrlistClass.__init__(self, field)
         if field:
@@ -752,7 +787,7 @@ class AddressList(AddrlistClass):
         return len(self.addresslist)
 
     def __str__(self):
-        return string.joinfields(map(dump_address_pair, self.addresslist),", ")
+        return ", ".join(map(dump_address_pair, self.addresslist))
 
     def __add__(self, other):
         # Set union
@@ -810,31 +845,33 @@ _daynames = ['mon', 'tue', 'wed', 'thu', 'fri', 'sat', 'sun']
 # zones.  RFC1123 recommends that numeric timezone indicators be used
 # instead of timezone names.
 
-_timezones = {'UT':0, 'UTC':0, 'GMT':0, 'Z':0, 
+_timezones = {'UT':0, 'UTC':0, 'GMT':0, 'Z':0,
               'AST': -400, 'ADT': -300,  # Atlantic (used in Canada)
               'EST': -500, 'EDT': -400,  # Eastern
               'CST': -600, 'CDT': -500,  # Central
               'MST': -700, 'MDT': -600,  # Mountain
               'PST': -800, 'PDT': -700   # Pacific
-              }    
+              }
 
 
 def parsedate_tz(data):
     """Convert a date string to a time tuple.
-    
+
     Accounts for military timezones.
     """
-    data = string.split(data)
-    if data[0][-1] in (',', '.') or string.lower(data[0]) in _daynames:
+    if not data:
+        return None
+    data = data.split()
+    if data[0][-1] in (',', '.') or data[0].lower() in _daynames:
         # There's a dayname here. Skip it
         del data[0]
     if len(data) == 3: # RFC 850 date, deprecated
-        stuff = string.split(data[0], '-')
+        stuff = data[0].split('-')
         if len(stuff) == 3:
             data = stuff + data[1:]
     if len(data) == 4:
         s = data[3]
-        i = string.find(s, '+')
+        i = s.find('+')
         if i > 0:
             data[3:] = [s[:i], s[i+1:]]
         else:
@@ -843,25 +880,25 @@ def parsedate_tz(data):
         return None
     data = data[:5]
     [dd, mm, yy, tm, tz] = data
-    mm = string.lower(mm)
+    mm = mm.lower()
     if not mm in _monthnames:
-        dd, mm = mm, string.lower(dd)
+        dd, mm = mm, dd.lower()
         if not mm in _monthnames:
             return None
     mm = _monthnames.index(mm)+1
     if mm > 12: mm = mm - 12
     if dd[-1] == ',':
         dd = dd[:-1]
-    i = string.find(yy, ':')
+    i = yy.find(':')
     if i > 0:
         yy, tm = tm, yy
     if yy[-1] == ',':
         yy = yy[:-1]
-    if yy[0] not in string.digits:
+    if not yy[0].isdigit():
         yy, tz = tz, yy
     if tm[-1] == ',':
         tm = tm[:-1]
-    tm = string.splitfields(tm, ':')
+    tm = tm.split(':')
     if len(tm) == 2:
         [thh, tmm] = tm
         tss = '0'
@@ -870,21 +907,21 @@ def parsedate_tz(data):
     else:
         return None
     try:
-        yy = string.atoi(yy)
-        dd = string.atoi(dd)
-        thh = string.atoi(thh)
-        tmm = string.atoi(tmm)
-        tss = string.atoi(tss)
-    except string.atoi_error:
+        yy = int(yy)
+        dd = int(dd)
+        thh = int(thh)
+        tmm = int(tmm)
+        tss = int(tss)
+    except ValueError:
         return None
-    tzoffset=None
-    tz=string.upper(tz)
-    if _timezones.has_key(tz):
-        tzoffset=_timezones[tz]
+    tzoffset = None
+    tz = tz.upper()
+    if tz in _timezones:
+        tzoffset = _timezones[tz]
     else:
-        try: 
-            tzoffset=string.atoi(tz)
-        except string.atoi_error: 
+        try:
+            tzoffset = int(tz)
+        except ValueError:
             pass
     # Convert a timezone offset into seconds ; -0500 -> -18000
     if tzoffset:
@@ -893,17 +930,17 @@ def parsedate_tz(data):
             tzoffset = -tzoffset
         else:
             tzsign = 1
-        tzoffset = tzsign * ( (tzoffset/100)*3600 + (tzoffset % 100)*60)
-    tuple = (yy, mm, dd, thh, tmm, tss, 0, 0, 0, tzoffset)
+        tzoffset = tzsign * ( (tzoffset//100)*3600 + (tzoffset % 100)*60)
+    tuple = (yy, mm, dd, thh, tmm, tss, 0, 1, 0, tzoffset)
     return tuple
 
 
 def parsedate(data):
     """Convert a time string to a time tuple."""
-    t=parsedate_tz(data)
-    if type(t)==type( () ):
+    t = parsedate_tz(data)
+    if type(t) == type( () ):
         return t[:9]
-    else: return t    
+    else: return t
 
 
 def mktime_tz(data):
@@ -919,11 +956,21 @@ def formatdate(timeval=None):
     """Returns time format preferred for Internet standards.
 
     Sun, 06 Nov 1994 08:49:37 GMT  ; RFC 822, updated by RFC 1123
+
+    According to RFC 1123, day and month names must always be in
+    English.  If not for that, this code could use strftime().  It
+    can't because strftime() honors the locale and could generated
+    non-English names.
     """
     if timeval is None:
         timeval = time.time()
-    return "%s" % time.strftime('%a, %d %b %Y %H:%M:%S GMT',
-                                time.gmtime(timeval))
+    timeval = time.gmtime(timeval)
+    return "%s, %02d %s %04d %02d:%02d:%02d GMT" % (
+            ["Mon", "Tue", "Wed", "Thu", "Fri", "Sat", "Sun"][timeval[6]],
+            timeval[2],
+            ["Jan", "Feb", "Mar", "Apr", "May", "Jun",
+             "Jul", "Aug", "Sep", "Oct", "Nov", "Dec"][timeval[1]-1],
+                                timeval[0], timeval[3], timeval[4], timeval[5])
 
 
 # When used as script, run a small test program.
@@ -960,8 +1007,8 @@ if __name__ == '__main__':
     print 'Lines:', n
     print '-'*70
     print 'len =', len(m)
-    if m.has_key('Date'): print 'Date =', m['Date']
-    if m.has_key('X-Nonsense'): pass
+    if 'Date' in m: print 'Date =', m['Date']
+    if 'X-Nonsense' in m: pass
     print 'keys =', m.keys()
     print 'values =', m.values()
     print 'items =', m.items()
