@@ -356,7 +356,7 @@ teedataobject_jumplink(teedataobject *tdo)
 {
 	if (tdo->nextlink == NULL)
 		tdo->nextlink = teedataobject_new(tdo->it);
-	Py_INCREF(tdo->nextlink);
+	Py_XINCREF(tdo->nextlink);
 	return tdo->nextlink;
 }
 
@@ -432,7 +432,7 @@ tee_next(teeobject *to)
 
 	if (to->index >= LINKCELLS) {
 		link = teedataobject_jumplink(to->dataobj);
-		Py_XDECREF(to->dataobj);
+		Py_DECREF(to->dataobj);
 		to->dataobj = (teedataobject *)link;
 		to->index = 0;
 	}
@@ -478,6 +478,12 @@ tee_fromiterable(PyObject *iterable)
 	if (to == NULL) 
 		goto done;
 	to->dataobj = (teedataobject *)teedataobject_new(it);
+	if (!to->dataobj) {
+		PyObject_GC_Del(to);
+		to = NULL;
+		goto done;
+	}
+
 	to->index = 0;
 	to->weakreflist = NULL;
 done:
@@ -618,6 +624,9 @@ cycle_new(PyTypeObject *type, PyObject *args, PyObject *kwds)
 	PyObject *saved;
 	cycleobject *lz;
 
+	if (!_PyArg_NoKeywords("cycle()", kwds))
+		return NULL;
+
 	if (!PyArg_UnpackTuple(args, "cycle", 1, 1, &iterable))
 		return NULL;
 
@@ -765,6 +774,9 @@ dropwhile_new(PyTypeObject *type, PyObject *args, PyObject *kwds)
 	PyObject *it;
 	dropwhileobject *lz;
 
+	if (!_PyArg_NoKeywords("dropwhile()", kwds))
+		return NULL;
+
 	if (!PyArg_UnpackTuple(args, "dropwhile", 2, 2, &func, &seq))
 		return NULL;
 
@@ -906,6 +918,9 @@ takewhile_new(PyTypeObject *type, PyObject *args, PyObject *kwds)
 	PyObject *it;
 	takewhileobject *lz;
 
+	if (!_PyArg_NoKeywords("takewhile()", kwds))
+		return NULL;
+
 	if (!PyArg_UnpackTuple(args, "takewhile", 2, 2, &func, &seq))
 		return NULL;
 
@@ -1048,6 +1063,9 @@ islice_new(PyTypeObject *type, PyObject *args, PyObject *kwds)
 	int numargs;
 	isliceobject *lz;
 
+	if (!_PyArg_NoKeywords("islice()", kwds))
+		return NULL;
+
 	numargs = PyTuple_Size(args);
 	if (!PyArg_ParseTuple(args, "OO|Ol:islice", &seq, &a1, &a2, &step))
 		return NULL;
@@ -1234,6 +1252,9 @@ starmap_new(PyTypeObject *type, PyObject *args, PyObject *kwds)
 	PyObject *it;
 	starmapobject *lz;
 
+	if (!_PyArg_NoKeywords("starmap()", kwds))
+		return NULL;
+
 	if (!PyArg_UnpackTuple(args, "starmap", 2, 2, &func, &seq))
 		return NULL;
 
@@ -1363,6 +1384,9 @@ imap_new(PyTypeObject *type, PyObject *args, PyObject *kwds)
 	imapobject *lz;
 	int numargs, i;
 
+	if (!_PyArg_NoKeywords("imap()", kwds))
+		return NULL;
+
 	numargs = PyTuple_Size(args);
 	if (numargs < 2) {
 		PyErr_SetString(PyExc_TypeError,
@@ -1542,6 +1566,9 @@ chain_new(PyTypeObject *type, PyObject *args, PyObject *kwds)
 	int i;
 	PyObject *ittuple;
 
+	if (!_PyArg_NoKeywords("chain()", kwds))
+		return NULL;
+
 	/* obtain iterators */
 	assert(PyTuple_Check(args));
 	ittuple = PyTuple_New(tuplesize);
@@ -1682,6 +1709,9 @@ ifilter_new(PyTypeObject *type, PyObject *args, PyObject *kwds)
 	PyObject *it;
 	ifilterobject *lz;
 
+	if (!_PyArg_NoKeywords("ifilter()", kwds))
+		return NULL;
+
 	if (!PyArg_UnpackTuple(args, "ifilter", 2, 2, &func, &seq))
 		return NULL;
 
@@ -1823,6 +1853,9 @@ ifilterfalse_new(PyTypeObject *type, PyObject *args, PyObject *kwds)
 	PyObject *it;
 	ifilterfalseobject *lz;
 
+	if (!_PyArg_NoKeywords("ifilterfalse()", kwds))
+		return NULL;
+
 	if (!PyArg_UnpackTuple(args, "ifilterfalse", 2, 2, &func, &seq))
 		return NULL;
 
@@ -1962,6 +1995,9 @@ count_new(PyTypeObject *type, PyObject *args, PyObject *kwds)
 	countobject *lz;
 	long cnt = 0;
 
+	if (!_PyArg_NoKeywords("count()", kwds))
+		return NULL;
+
 	if (!PyArg_ParseTuple(args, "|l:count", &cnt))
 		return NULL;
 
@@ -2058,6 +2094,9 @@ izip_new(PyTypeObject *type, PyObject *args, PyObject *kwds)
 	PyObject *result;
 	int tuplesize = PySequence_Length(args);
 
+	if (!_PyArg_NoKeywords("izip()", kwds))
+		return NULL;
+
 	/* args must be a tuple */
 	assert(PyTuple_Check(args));
 
@@ -2238,6 +2277,9 @@ repeat_new(PyTypeObject *type, PyObject *args, PyObject *kwds)
 	PyObject *element;
 	long cnt = -1;
 
+	if (!_PyArg_NoKeywords("repeat()", kwds))
+		return NULL;
+
 	if (!PyArg_ParseTuple(args, "O|l:repeat", &element, &cnt))
 		return NULL;
 
@@ -2418,6 +2460,8 @@ inititertools(void)
 
 	teedataobject_type.ob_type = &PyType_Type;
 	m = Py_InitModule3("itertools", module_methods, module_doc);
+	if (m == NULL)
+		return;
 
 	for (i=0 ; typelist[i] != NULL ; i++) {
 		if (PyType_Ready(typelist[i]) < 0)
