@@ -1,7 +1,7 @@
 /*
  /  Author: Sam Rushing <rushing@nightmare.com>
  /  Hacked for Unix by AMK
- /  $Id: mmapmodule.c 42245 2006-02-05 06:00:54Z neal.norwitz $
+ /  $Id: mmapmodule.c 52017 2006-09-27 19:17:32Z andrew.kuchling $
 
  / mmapmodule.cpp -- map a view of a file into memory
  /
@@ -1093,6 +1093,8 @@ PyMODINIT_FUNC
 	mmap_object_type.ob_type = &PyType_Type;
 
 	module = Py_InitModule ("mmap", mmap_functions);
+	if (module == NULL)
+		return;
 	dict = PyModule_GetDict (module);
 	mmap_module_error = PyExc_EnvironmentError;
 	Py_INCREF(mmap_module_error);
