@@ -662,6 +662,8 @@ initselect(void)
 {
 	PyObject *m;
 	m = Py_InitModule3("select", select_methods, module_doc);
+	if (m == NULL)
+		return;
 
 	SelectError = PyErr_NewException("select.error", NULL, NULL);
 	Py_INCREF(SelectError);
