@@ -130,7 +130,10 @@ dl_funcptr _PyImport_GetDynLoadFunc(const char *fqname, const char *shortname,
 	handle = dlopen(pathname, dlopenflags);
 
 	if (handle == NULL) {
-		PyErr_SetString(PyExc_ImportError, dlerror());
+		char *error = dlerror();
+		if (error == NULL)
+			error = "unknown dlopen() error";
+		PyErr_SetString(PyExc_ImportError, error);
 		return NULL;
 	}
 	if (fp != NULL && nhandles < 128)
