@@ -144,6 +144,9 @@ vgetargs1(PyObject *args, char *format, va_list *p_va, int compat)
 			if (level == 0)
 				max++;
 			level++;
+			if (level >= 30)
+				Py_FatalError("too many tuple nesting levels "
+					      "in argument format string");
 			break;
 		case ')':
 			if (level == 0)
@@ -287,8 +290,8 @@ seterror(int iarg, char *msg, int *levels, char *fname, char *message)
 				      "argument %d", iarg);
 			i = 0;
 			p += strlen(p);
-			while (levels[i] > 0 && (int)(p-buf) < 220) {
-				PyOS_snprintf(p, sizeof(buf) - (buf - p),
+			while (levels[i] > 0 && i < 32 && (int)(p-buf) < 220) {
+				PyOS_snprintf(p, sizeof(buf) - (p - buf),
 					      ", item %d", levels[i]-1);
 				p += strlen(p);
 				i++;
@@ -374,6 +377,13 @@ converttuple(PyObject *arg, char **p_format, va_list *p_va, int *levels,
 		char *msg;
 		PyObject *item;
 		item = PySequence_GetItem(arg, i);
+ 		if (item == NULL) {
+ 			PyErr_Clear();
+ 			levels[0] = i+1;
+ 			levels[1] = 0;
+ 			strncpy(msgbuf, "is not retrievable", bufsize);
+ 			return msgbuf;
+ 		}
 		msg = convertitem(item, &format, p_va, levels+1, msgbuf,
 				  bufsize, freelist);
 		/* PySequence_GetItem calls tp->sq_item, which INCREFs */
@@ -1361,6 +1371,7 @@ vgetargskeywords(PyObject *args, PyObject *keywords, char *format,
 		else {
 			msg = skipitem(&format, p_va);
 			if (msg) {
+				levels[0] = 0;
 				seterror(i+1, msg, levels, fname, message);
 				return cleanreturn(0, freelist);
 			}
